from PyQt5.QtWidgets import QWidget, QMessageBox
from PyQt5.QtCore import pyqtSlot
from controllers.base_controller import BaseController
from controllers.requirement_dialog import NewRequirementDialog, RequirementEditDialog
from model.requirements import Requirement
from utils.distribution_maker import DistributionMaker, ImpossibleDistributionError
from utils.requirements_table import RequirementsTable
from windows import distributionsetup


class EmptyGroupError(Exception):
    pass


class DistributionSetup(BaseController, QWidget, distributionsetup.Ui_Form):
    def __init__(self, main_window, group, parent=None):
        """
        :param main_window: main window controller 
        :param group: (Group) that is being distributed
        :param parent: parent widget for qt
        """
        if len(group.members) == 0:
            raise EmptyGroupError()

        super(DistributionSetup, self).__init__(parent)
        self.setupUi(self)
        self.main_window = main_window
        self.tableRequirements = RequirementsTable(
            2,
            ({'text': "", 'tooltip': self.tr("Upravit"), 'icon': ":/icons/edit.svg"},
             {'text': "", 'tooltip': self.tr("Smazat omezení"), 'icon': ":/icons/minus.svg"},)
        )
        self.placeTable.insertWidget(1, self.tableRequirements)
        self.tableRequirements.btnClicked.connect(self.onTableRequirementsBtnClicked)
        self.tableRequirements.itemDoubleClicked.connect(self.editRequirement)
        self.group = group

        # set up header
        self.lblGroupName.setText(self.group.name)
        self.lblMemberCount.setText('(' + str(self.group.member_count) + ' členů)')

        # set up comboPrefer
        self.comboPrefer.setItemData(0, None)
        self.comboPrefer.setItemData(1, DistributionMaker.PREFER_SAME_TEAMS)
        self.comboPrefer.setItemData(2, DistributionMaker.PREFER_DIFFERENT_TEAMS)

        # set up spinners
        self.spinTeamCount.setMinimum(1)
        self.spinTeamCount.setMaximum(self.group.member_count)
        self.spinMinTeamSize.setMinimum(0)
        self.spinMinTeamSize.setMaximum(self.group.member_count)
        self.spinMaxTeamSize.setMinimum(1)
        self.spinMaxTeamSize.setMaximum(self.group.member_count)
        self.spinMaxTeamSize.setValue(self.group.member_count)

        # set up checkboxes
        self.cbIgnoreTeamCount.stateChanged.connect(self.spinTeamCount.setDisabled)
        self.cbIgnoreMinSize.stateChanged.connect(self.spinMinTeamSize.setDisabled)
        self.cbIgnoreMaxSize.stateChanged.connect(self.spinMaxTeamSize.setDisabled)

        self.btnAddRequirement.clicked.connect(self.showRequirementDialog)
        self.btnDistribute.clicked.connect(self.distribute)

    @pyqtSlot()
    def showRequirementDialog(self):
        """
        display dialog that allows creating of requirement
        then, after user creates the requirement insert it into requirements table
        """
        dialog = NewRequirementDialog(self.group, self.tableRequirements.model.sourceModel().requirements, parent=self)
        dialog.exec_()
        if dialog.result() == dialog.Accepted:
            if dialog.requirement is not None:
                self.tableRequirements.insertRequirement(dialog.requirement)

    @pyqtSlot(Requirement, int)
    def onTableRequirementsBtnClicked(self, requirement, btn_nr):
        """
        check which button was clicked and act accordingly
        :param requirement: selected requirement
        :param btn_nr: column of clicked button (0 is the first column with buttons, not first column in table)
        """
        if btn_nr == 0:
            # it's btn "edit"
            self.editRequirement(requirement)
        elif btn_nr == 1:
            # it's btn "delete"
            self.deleteRequirement(requirement)

    def editRequirement(self, requirement):
        """
        remove requirement from requirement table,
        open dialog that allows editting of requirement 
        then, after the dialog is done, reinsert the requirement into table
        :param requirement: requirement that we're editting
        """
        self.tableRequirements.model.sourceModel().removeRequirement(requirement)
        dialog = RequirementEditDialog(self.group, requirement)
        dialog.exec_()
        if dialog.result() == dialog.Accepted:
            if dialog.requirement is not None:
                self.tableRequirements.insertRequirement(dialog.requirement)
        else:
            self.tableRequirements.insertRequirement(requirement)

    def deleteRequirement(self, requirement):
        """
        show confirmation dialog, then delete requirement
        :param requirement: requirement to be deleted
        """
        msg = QMessageBox(self)
        msg.setIcon(QMessageBox.Question)
        msg.setWindowTitle("Odebrat omezení?")
        msg.setText("Chcete odebrat toto omezení?")
        msg.setStandardButtons(QMessageBox.Ok | QMessageBox.Cancel)
        if msg.exec() == QMessageBox.Ok:
            self.tableRequirements.model.sourceModel().removeRequirement(requirement)

    def distribute(self):
        """
        create distribution with set parameters and requirements in requirements table
        if successful, move to DistributionOverview, else show error message
        """
        try:
            dist_maker = DistributionMaker(
                self.group,
                self.tableRequirements.getAllRequirements(),
                team_count=self.spinTeamCount.value() if not self.cbIgnoreTeamCount.isChecked() else None,
                min_team_size=self.spinMinTeamSize.value() if not self.cbIgnoreMinSize.isChecked() else None,
                max_team_size=self.spinMaxTeamSize.value() if not self.cbIgnoreMaxSize.isChecked() else None,
                prefer=self.comboPrefer.currentData()
            )
            self.main_window.goToDistributionOverview(dist_maker.distribute(), self.group)
        except ImpossibleDistributionError as e:
            msg = QMessageBox(self)
            msg.setIcon(QMessageBox.Critical)
            msg.setWindowTitle(self.tr("Chyba"))
            msg.setText(self.tr("Rozdělení s požadovanými parametry nelze vytvořit!"))
            msg.setStandardButtons(QMessageBox.Ok)
            msg.exec_()
